package pps.core.space.service;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.text.CharSequenceUtil;
import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.github.pagehelper.PageInfo;
import com.github.pagehelper.page.PageMethod;
import org.apache.commons.lang3.StringUtils;
import pps.cloud.base.service.data.base_price_strategy_detail.GetBasePriceStrategyDetailInput;
import pps.cloud.prediction.service.data.plant_predicted_power_data.DynamicQueryPlantPredictedPowerInput;
import pps.core.common.constant.BusinessConstant;
import pps.core.common.entity.BaseModel;
import pps.core.space.entity.*;
import pps.core.space.mapper.*;
import pps.core.space.service.data.SpaceOptimizeDateDuration;
import pps.core.space.service.data.SpaceOptimizeDateOutput;
import pps.core.space.service.data.space_optimize_short_period.GetSpaceOptimizeShortPeriodViewInput;
import pps.core.space.service.data.space_optimize_short_period.GetSpaceOptimizeShortPeriodViewOutput;
import pps.core.space.service.data.space_optimize_short_period.QuerySpaceOptimizeShortPeriodViewInput;
import pps.core.space.service.data.space_optimize_short_period.QuerySpaceOptimizeShortPeriodViewOutput;
import pps.core.space.service.data.space_optimize_short_wellhead.GetSpaceOptimizeShortWellheadInput;
import pps.core.space.service.data.space_optimize_short_wellhead.GetSpaceOptimizeShortWellheadOutput;
import pps.core.space.service.data.space_optimize_short_wellhead.QuerySpaceOptimizeShortWellheadViewInput;
import pps.core.space.service.data.space_optimize_short_wellhead.QuerySpaceOptimizeShortWellheadViewOutput;
import pps.core.space.utils.ServiceUtil;
import xstartup.annotation.XService;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.base.util.XCopyUtils;
import xstartup.data.XListResult;
import xstartup.data.XPageResult;
import xstartup.data.XSingleResult;
import xstartup.error.XError;
import xstartup.feature.api.annotation.XApiGet;

import java.util.*;
import java.util.stream.Collectors;

/**
 * 极短期间开优化模块
 *
 * @author ZWT
 * @date 2023/09/07
 */
@XService
@XText("极短期间开优化模块")
public class SpaceOptimizeShortPeriodService extends SpaceOptimizeBaseService {

    /**
     * 极短期间开开优化--分页列表
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XPageResult}<{@link QuerySpaceOptimizeShortPeriodViewOutput}>
     */
    @XApiGet
    @XText("极短期间开优化--分页列表")
    public XPageResult<QuerySpaceOptimizeShortPeriodViewOutput> querySpaceOptimizeShortPeriodView(XContext context, QuerySpaceOptimizeShortPeriodViewInput input) {
        SpaceOptimizeShortPeriodViewMapper mapper = context.getBean(SpaceOptimizeShortPeriodViewMapper.class);
        SpaceOptimizeShortPeriodView record = XCopyUtils.copyNewObject(input, SpaceOptimizeShortPeriodView.class);
        PageMethod.startPage(input.getPage(), input.getLimit());
        List<SpaceOptimizeShortPeriodView> list = mapper.selectList(record);
        PageInfo<SpaceOptimizeShortPeriodView> pageInfo = new PageInfo<>(list);
        List<QuerySpaceOptimizeShortPeriodViewOutput> outputs = XCopyUtils.copyNewList(pageInfo.getList(), QuerySpaceOptimizeShortPeriodViewOutput.class);
        if (CollUtil.isNotEmpty(outputs)) {
            Map<String, String> gridTypeMap = ServiceUtil.getDictMap(context, BusinessConstant.GRID_TYPE);
            Map<String, String> ouMap = ServiceUtil.getOuMapByOuIds(context, outputs.stream()
                    .map(QuerySpaceOptimizeShortPeriodViewOutput::getOuId)
                    .distinct()
                    .collect(Collectors.toList())
            );
            String lineName = ServiceUtil.getPowerLineNameById(context, input.getLineId());
            outputs.forEach(output -> {
                output.setLineName(lineName);
                output.setOuName(ouMap.get(output.getOuId()));
                output.setGridTypeName(gridTypeMap.get(output.getGridTypeKey()));
                output.setOptimizeStateValue(output.getOptimizeState().equals(0) ? BusinessConstant.OPTIMIZED_C : BusinessConstant.NO_OPTIMIZATION_C);
            });
        }
        return XPageResult.success(outputs, input, pageInfo.getTotal());
    }

    /**
     * 极短期间开开优化--详情
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XSingleResult}<{@link GetSpaceOptimizeShortPeriodViewOutput}>
     */
    @XApiGet
    @XText("极短期间开优化--详情")
    public XSingleResult<GetSpaceOptimizeShortPeriodViewOutput> getSpaceOptimizeShortPeriodView(XContext context, GetSpaceOptimizeShortPeriodViewInput input) {
        SpaceOptimizeShortPeriodMapper mapper = context.getBean(SpaceOptimizeShortPeriodMapper.class);
        String shortPeriodId = input.getId();
        SpaceOptimizeShortPeriodEnt period = mapper.selectById(shortPeriodId);
        if (Objects.isNull(period)) {
            return XSingleResult.error(context, XError.NotFound);
        }
        GetSpaceOptimizeShortPeriodViewOutput output = XCopyUtils.copyNewObject(period, GetSpaceOptimizeShortPeriodViewOutput.class);
        //查间开制度详情
        SpaceInstitutionDetailMapper detailMapper = context.getBean(SpaceInstitutionDetailMapper.class);
        SpaceInstitutionDetailEnt detailEnt = detailMapper.selectById(period.getInstitutionId());
        List<SpaceOptimizeDateOutput> optimizeDateList = ServiceUtil.getOptimizeDateList(period.getCreateTime(), period.getOptimizeDate());
        output.setDateList(optimizeDateList);
        if (StringUtils.equals(String.valueOf(BusinessConstant.ONE), Optional.ofNullable(detailEnt.getGridTypeKey())
                .orElse(String.valueOf(BusinessConstant.ZERO)))) {
            int month;
            if (CollUtil.isNotEmpty(optimizeDateList)) {
                month = DateUtil.month(optimizeDateList.get(optimizeDateList.size() - 1).getDateTime());
            } else {
                Calendar calendar = Calendar.getInstance();
                calendar.setTime(period.getOptimizeDate());
                month = calendar.get(Calendar.MONTH);
            }
            output.setStrategyDetailList(ServiceUtil.getStrategyDetailList(context,
                    GetBasePriceStrategyDetailInput.builder()
                            .lineId(period.getLineId())
                            .strategyMonth(String.valueOf(month + 1))
                            .build()));
        } else {
            output.setStrategyDetailList(new ArrayList<>(0));
        }
        output.setGridTypeKey(detailEnt.getGridTypeKey());
        //查井口列表
        SpaceOptimizeShortWellheadMapper wellheadMapper = context.getBean(SpaceOptimizeShortWellheadMapper.class);
        List<SpaceOptimizeShortWellheadEnt> wellheadList = wellheadMapper.selectList(new LambdaQueryWrapper<SpaceOptimizeShortWellheadEnt>()
                .eq(SpaceOptimizeShortWellheadEnt::getShortPeriodId, shortPeriodId)
                .eq(BaseModel::getIsDeleted, BusinessConstant.ONE)
                .orderByAsc(SpaceOptimizeShortWellheadEnt::getStartSeq)
        );
        if (CollUtil.isNotEmpty(wellheadList)) {
            List<GetSpaceOptimizeShortWellheadOutput> wellheadOutputs = XCopyUtils.copyNewList(wellheadList, GetSpaceOptimizeShortWellheadOutput.class);
            //查井口配置列表
            SpaceOptimizeShortDurationMapper durationMapper = context.getBean(SpaceOptimizeShortDurationMapper.class);
            List<SpaceOptimizeShortDurationEnt> durationList = durationMapper.selectList(new LambdaQueryWrapper<SpaceOptimizeShortDurationEnt>()
                    .eq(SpaceOptimizeShortDurationEnt::getShortPeriodId, shortPeriodId)
                    .eq(BaseModel::getIsDeleted, BusinessConstant.ONE)
                    .orderByAsc(SpaceOptimizeShortDurationEnt::getIsOptimize, SpaceOptimizeShortDurationEnt::getOpenWellTime)
            );
            if (CollUtil.isNotEmpty(durationList)) {
                List<SpaceOptimizeDateDuration> durationOutputs = XCopyUtils.copyNewList(durationList, SpaceOptimizeDateDuration.class);
                Map<String, List<SpaceOptimizeDateDuration>> collect = new HashMap<>(16);
                for (SpaceOptimizeDateDuration durationOutput : durationOutputs) {
                    if (StringUtils.isEmpty(durationOutput.getGenerationTypeKey())) {
                        durationOutput.setGenerationTypeKey(BusinessConstant.PHOTOVOLTAIC);
                    }
                    List<SpaceOptimizeDateDuration> tmpList = collect.get(durationOutput.getWellheadId());
                    if (CollUtil.isEmpty(tmpList)) {
                        tmpList = new ArrayList<>(16);
                        tmpList.add(durationOutput);
                        collect.put(durationOutput.getWellheadId(), tmpList);
                    } else {
                        tmpList.add(durationOutput);
                    }
                }
                wellheadOutputs.forEach(w -> {
                    if (collect.containsKey(w.getWellheadId())) {
                        Map<Integer, List<SpaceOptimizeDateDuration>> listMap = collect.get(w.getWellheadId()).stream()
                                .collect(Collectors.groupingBy(SpaceOptimizeDateDuration::getIsOptimize));
                        w.setOptimizeList(ServiceUtil.getAllDateDuration(listMap.get(0)));
                        w.setInitializeList(ServiceUtil.getAllDateDuration(listMap.get(1)));
                    } else {
                        w.setOptimizeList(new ArrayList<>(0));
                        w.setInitializeList(new ArrayList<>(0));
                    }
                });
            }
            output.setWellheadList(wellheadOutputs);
        } else {
            output.setWellheadList(new ArrayList<>(0));
        }
        //查功率预测信息
        output.setPowerGenerationList(
                super.getAveragePowerGenerationHourList(context, DynamicQueryPlantPredictedPowerInput.builder()
                        .plantIds(super.getOptimizeLineRelation(context, CollUtil.newArrayList(detailEnt))
                                .getPlantIdsByLineIdMap()
                                .get(detailEnt.getLineId()))
                        .startTime(DateUtil.beginOfDay(period.getCreateTime()).toString())
                        .endTime(DateUtil.beginOfDay(period.getOptimizeDate()).toString())
                        .dateType(BusinessConstant.ONE)
                        .build())
        );
        return XSingleResult.success(output);
    }

    /**
     * 获取当日井口优化列表
     * GET /space/space-optimize-short-period/get-optimize-list
     * 接口ID：158457157
     * 接口地址：https://app.apifox.com/project/3196988/apis/api-158457157
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XListResult}<{@link GetSpaceOptimizeShortWellheadOutput}>
     */
    @XApiGet
    @XText("极短期间开优化--获取当日井口优化列表")
    public XListResult<GetSpaceOptimizeShortWellheadOutput> getOptimizeList(XContext context, GetSpaceOptimizeShortWellheadInput input) {
        DateTime now = DateUtil.beginOfDay(DateTime.now());
        String lineId = input.getLineId();
        String wellheadId = input.getWellheadId();
        SpaceOptimizeShortPeriodMapper mapper = context.getBean(SpaceOptimizeShortPeriodMapper.class);
        SpaceOptimizeShortPeriodEnt periodEnt = mapper.selectOne(new LambdaQueryWrapper<SpaceOptimizeShortPeriodEnt>()
                .eq(SpaceOptimizeShortPeriodEnt::getOptimizeDate, now)
                .eq(SpaceOptimizeShortPeriodEnt::getLineId, lineId)
                .last("limit 1")
        );
        if (ObjectUtil.isEmpty(periodEnt)) {
            return XListResult.success(new ArrayList<>(0));
        }
        SpaceOptimizeShortWellheadMapper wellheadMapper = context.getBean(SpaceOptimizeShortWellheadMapper.class);
        List<SpaceOptimizeShortWellheadEnt> wellheadList = wellheadMapper.selectList(new LambdaQueryWrapper<SpaceOptimizeShortWellheadEnt>()
                .eq(SpaceOptimizeShortWellheadEnt::getShortPeriodId, periodEnt.getId())
                .eq(CharSequenceUtil.isNotBlank(wellheadId), SpaceOptimizeShortWellheadEnt::getWellheadId, wellheadId)
        );
        if (CollUtil.isEmpty(wellheadList)) {
            return XListResult.success(new ArrayList<>(0));
        }
        List<GetSpaceOptimizeShortWellheadOutput> wellheadOutputs = XCopyUtils.copyNewList(wellheadList, GetSpaceOptimizeShortWellheadOutput.class);
        SpaceOptimizeShortDurationMapper durationMapper = context.getBean(SpaceOptimizeShortDurationMapper.class);
        List<SpaceOptimizeShortDurationEnt> durationList = durationMapper.selectList(new LambdaQueryWrapper<SpaceOptimizeShortDurationEnt>()
                .eq(SpaceOptimizeShortDurationEnt::getShortPeriodId, periodEnt.getId())
                .eq(BaseModel::getIsDeleted, BusinessConstant.ONE)
                .eq(SpaceOptimizeShortDurationEnt::getIsOptimize, BusinessConstant.ZERO)
                .eq(CharSequenceUtil.isNotBlank(wellheadId), SpaceOptimizeShortDurationEnt::getWellheadId, wellheadId)
                .orderByAsc(SpaceOptimizeShortDurationEnt::getIsOptimize, SpaceOptimizeShortDurationEnt::getOpenWellTime)
        );
        List<SpaceOptimizeDateDuration> durationOutputs = XCopyUtils.copyNewList(durationList, SpaceOptimizeDateDuration.class);
        Map<String, List<SpaceOptimizeDateDuration>> collect = durationOutputs.stream()
                .collect(Collectors.groupingBy(SpaceOptimizeDateDuration::getWellheadId));
        for (GetSpaceOptimizeShortWellheadOutput wellhead : wellheadOutputs) {
            wellhead.setOptimizeList(ServiceUtil.getAllDateDuration(collect.get(wellhead.getWellheadId())));
        }
        return XListResult.success(wellheadOutputs);
    }

    /**
     * 先导实验井间开制度
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XPageResult}<{@link QuerySpaceOptimizeShortWellheadViewOutput}>
     */
    @XApiGet
    @XText("极短期间开优化--先导实验井间开制度")
    public XPageResult<QuerySpaceOptimizeShortWellheadViewOutput> getDailySystemDisplay(XContext context, QuerySpaceOptimizeShortWellheadViewInput input) {
        SpaceOptimizeShortDurationViewMapper mapper = context.getBean(SpaceOptimizeShortDurationViewMapper.class);
        PageMethod.startPage(input.getPage(), input.getLimit());
        List<SpaceOptimizeShortDurationView> list = mapper.selectDailySystemDisplay(SpaceOptimizeShortDurationView.builder()
                .queryDate(DateUtil.beginOfDay(DateUtil.date()))
                .ouId(input.getStationId())
                .build());
        PageInfo<SpaceOptimizeShortDurationView> pageInfo = new PageInfo<>(list);
        return XPageResult.success(XCopyUtils.copyNewList(pageInfo.getList(), QuerySpaceOptimizeShortWellheadViewOutput.class),
                input, pageInfo.getTotal());
    }
}